@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import asyncio
 import importlib
+import struct
 from collections import defaultdict
 
 from typing import Any, Callable, Optional, Type, cast
@@ -199,7 +200,7 @@ class Module(object):
                     f"Device {name!r} already exists in module {self.name!r}."
                 )
 
-            assert address, "address cannot be None"
+            assert address is not None, "address cannot be None"
 
             device = device_class(self, name, address, **kwargs)
 
@@ -236,7 +237,9 @@ class Device(object):
     name
         The name of the device. It is treated as case-insensitive.
     address
-        The address of the device.
+        The address of the device. This is the address passed to ``pymodbus``
+        (i.e., the one encoded in the TCP packet) so it should have any offset
+        already removed.
     mode
         The type of object (``coil``, ``discrete``, ``input_register``, or
         ``holding_register``). If `None`, uses the mode from the module.
@@ -254,6 +257,10 @@ class Device(object):
         A description of the purpose of the device.
     units
         The units of the values returned, if an adaptor is used.
+    data_type
+        The data type for the raw values read. If not set, assumes unsigned
+        integer. The format is the same as ``struct`` data types. For example,
+        if reading a signed integer use ``h``.
     adaptor
         The adaptor to be used to convert read registers to physical values.
         It can be a string indicating one of the provided :ref:`adaptors
@@ -282,6 +289,7 @@ class Device(object):
         offset: float = 0.0,
         units: Optional[str] = None,
         category: Optional[str] = None,
+        data_type: Optional[str] = None,
         adaptor: Optional[str | dict | Callable] = None,
         adaptor_extra_params: tuple[Any] = tuple(),
     ):
@@ -296,6 +304,7 @@ class Device(object):
         self.units = units
         self.category = category
         self.offset = offset
+        self.data_type = data_type
         self.adaptor = self._parse_adaptor(adaptor)
         self._adaptor_extra_params = adaptor_extra_params
 
@@ -396,18 +405,23 @@ class Device(object):
 
         protocol = self.client.protocol
 
+        raw_type: str = ""
         if self.mode == "coil":
             reader = protocol.read_coils
+            raw_type = "?"
         elif self.mode == "discrete":
             reader = protocol.read_discrete_inputs
+            raw_type = "?"
         elif self.mode == "input_register":
             reader = protocol.read_input_registers
+            raw_type = "H"
         elif self.mode == "holding_register":
             reader = protocol.read_holding_registers
+            raw_type = "H"
         else:
             raise DriftError(f"invalid mode {self.mode!r}.")
 
-        resp = await reader(self.address - 40001, count=1)
+        resp = await reader(self.address, count=1)
 
         if resp.function_code > 0x80:
             raise DriftError(
@@ -422,6 +436,9 @@ class Device(object):
             if self.channel is not None:
                 value = (value & (1 << self.channel)) > 0
 
+        if self.data_type is not None:
+            value = struct.unpack(self.data_type, struct.pack(f"={raw_type}", value))[0]
+
         if not adapt:
             return value
 
@@ -480,16 +497,16 @@ class Device(object):
         protocol = self.client.protocol
 
         if self.mode == "coil":
-            resp = await protocol.write_coil(self.address - 40001, value)
+            resp = await protocol.write_coil(self.address, value)
         else:
             if self.channel is not None:
-                resp = await protocol.read_holding_registers(self.address - 40001)
+                resp = await protocol.read_holding_registers(self.address)
                 current_value = resp.registers[0]
                 if value is True or value > 0:
                     value = current_value | (1 << self.channel)
                 else:
                     value = current_value & (~(1 << self.channel))
-            resp = await protocol.write_register(self.address - 40001, value)
+            resp = await protocol.write_register(self.address, value)
 
         if resp.function_code > 0x80:
             raise DriftError(
@@ -567,7 +584,7 @@ class Drift(object):
 
         drift = Drift('localhost')
         async with drift:
-            coil = drift.client.protocol.read_coils(40001, count=1)
+            coil = drift.client.protocol.read_coils(1, count=1)
 
     This is not needed when using `.Device.read` or `.Device.write`, which
     handle the connection to the server.
@@ -738,7 +755,7 @@ class Drift(object):
         return values
 
     @classmethod
-    def from_config(cls, config: str | dict):
+    def from_config(cls, config: str | dict, **kwargs):
         """Loads a Drift from a dictionary or YAML file.
 
         Parameters
@@ -756,7 +773,7 @@ class Drift(object):
         address = config["address"]
         port = config.get("port", 502)
 
-        new_drift = cls(address, port)
+        new_drift = cls(address, port, **kwargs)
 
         for module in config.get("modules", {}):
 
